// Sorry, this is not header file. Hardware interface is included here!

#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 1000000UL  // 1 MHz
#include <util/delay.h>

// ******** Bit manipulation definitions ***********
// Power ON/OFF switch
#define POWER_OFF asm ("cbi 0x12,5" ::)
#define POWER_ON asm ("sbi 0x12,5" ::)
// Mute ON/OFF
#define MUTE_OFF asm ("cbi 0x12,6" ::)
#define MUTE_ON asm ("sbi 0x12,6" ::)
// Stereo ON/OFF
#define STEREO_OFF asm ("sbi 0x12,4" ::)
#define STEREO_ON asm ("cbi 0x12,4" ::)
// PLL frequency synthetizer bus
#define PLL_DLEN_0 asm ("cbi 0x18,0" ::)
#define PLL_DLEN_1 asm ("sbi 0x18,0" ::)
#define PLL_DATA_0 asm ("cbi 0x18,1" ::)
#define PLL_DATA_1 asm ("sbi 0x18,1" ::)
#define PLL_CLK_0 asm ("cbi 0x18,2" ::)
#define PLL_CLK_1 asm ("sbi 0x18,2" ::)
// LCD 2x8 chars, Write only, 4 bits bus
#define DISP_E_0 asm ("cbi 0x12,7" ::)  // LCD signals definition
#define DISP_E_1 asm ("sbi 0x12,7" ::)
#define DISP_INST asm ("cbi 0x18,1" ::)
#define DISP_DATA asm ("sbi 0x18,1" ::)
#define DISP_DB7_0 asm ("cbi 0x18,5" ::)
#define DISP_DB7_1 asm ("sbi 0x18,5" ::)
#define DISP_DB6_0 asm ("cbi 0x18,4" ::)
#define DISP_DB6_1 asm ("sbi 0x18,4" ::)
#define DISP_DB5_0 asm ("cbi 0x18,3" ::)
#define DISP_DB5_1 asm ("sbi 0x18,3" ::)
#define DISP_DB4_0 asm ("cbi 0x18,2" ::)
#define DISP_DB4_1 asm ("sbi 0x18,2" ::)
// Delay for display instructions
#define Delay50u _delay_loop_1(15)
#define Delay2ms _delay_loop_2(500)

// ************** Display ******************
// Write instruction code to display
void WriteIns(uint8_t code)
{
	DISP_INST;	// Set display for instruction
	if(code & 0x80) DISP_DB7_1; else DISP_DB7_0;
	if(code & 0x40) DISP_DB6_1; else DISP_DB6_0;
	if(code & 0x20) DISP_DB5_1; else DISP_DB5_0;
	if(code & 0x10) DISP_DB4_1; else DISP_DB4_0;
	// Write upper nibble
	DISP_E_1;
	DISP_E_0;
	if(code & 0x08) DISP_DB7_1; else DISP_DB7_0;
	if(code & 0x04) DISP_DB6_1; else DISP_DB6_0;
	if(code & 0x02) DISP_DB5_1; else DISP_DB5_0;
	if(code & 0x01) DISP_DB4_1; else DISP_DB4_0;
	// Write lower nibble
	DISP_E_1;
	DISP_E_0;
	DISP_DB7_0;
	DISP_DB6_0;
	DISP_DB5_0;
	DISP_DB4_0;
	Delay50u;
}

// Write data (char) to display
void WriteChar(uint8_t code)
{
	DISP_DATA;	// Set display for data
	if(code & 0x80) DISP_DB7_1; else DISP_DB7_0;
	if(code & 0x40) DISP_DB6_1; else DISP_DB6_0;
	if(code & 0x20) DISP_DB5_1; else DISP_DB5_0;
	if(code & 0x10) DISP_DB4_1; else DISP_DB4_0;
	// Write upper nibble
	DISP_E_1;
	DISP_E_0;
	if(code & 0x08) DISP_DB7_1; else DISP_DB7_0;
	if(code & 0x04) DISP_DB6_1; else DISP_DB6_0;
	if(code & 0x02) DISP_DB5_1; else DISP_DB5_0;
	if(code & 0x01) DISP_DB4_1; else DISP_DB4_0;
	// Write lower nibble
	DISP_E_1;
	DISP_E_0;
	DISP_INST;
	DISP_DB7_0;
	DISP_DB6_0;
	DISP_DB5_0;
	DISP_DB4_0;
	Delay50u;
}

void InitDisplay()
{
	DISP_E_0;
	Delay2ms;
// Set 4 bit communication
	DISP_E_0;
	DISP_INST;
	DISP_DB7_0;
	DISP_DB6_0;
	DISP_DB5_1;
	DISP_DB4_0;
	DISP_E_1;
	DISP_E_0;
	Delay2ms;
// Set 2 line, 4 bit communication, font 5x7
	WriteIns(0x28);
// Display ON, Cursor OFF, Blink on cursor OFF
	WriteIns(0x0C);
// Reset display, Set cursor 0,0 position
	WriteIns(0x01);
	Delay2ms;
// Set cursor increment
	WriteIns(0x06);
}

// Write null terminated string to display
void WriteText(char *character)
{
   while(*character) {
   		WriteChar(*character);
		character++;
	}
}

// Set cursor position
void SetDispPos(uint8_t Xpos, uint8_t Ypos)
{
	if((Xpos>7) || (Ypos>1)) return;
	if(Ypos) WriteIns(Xpos | 0xC0);
		else WriteIns(Xpos | 0x80);
}

// Clear display, set position 0,0
void ClrDisp()
{
// Reset display, Set cursor 0,0 position
	WriteIns(0x01);
	Delay2ms;
}

// ************** PLL synthetizer ******************
void SetFrequencyLM(uint16_t Freq)
{
	uint8_t i;
	// Set reasonable FM frequencies only
	if((Freq > 10800) || (Freq < 8750)) return;
	// Add 10.7 MHz for IF
	Freq += 1070;
	// Calculate how many 50 kHz units
	Freq /= 5;
	// Set initial conditions
	PLL_DLEN_0;
	PLL_DATA_0;
	PLL_CLK_0;
	_delay_loop_1(5);
	// Start transmition
	PLL_DLEN_1;
	// Loop for 14 bits - LSB first
	for(i=0;i<14;i++) {
		if(Freq & 0x0001) PLL_DATA_1; else PLL_DATA_0;
		Freq = Freq >> 1;
		_delay_loop_1(2);
		PLL_CLK_1;
		_delay_loop_1(2);
		PLL_CLK_0;
	}
	// Set mode of operation
	Freq = 0x0310;
	for(i=0;i<10;i++) {
		if(Freq & 0x0001) PLL_DATA_1; else PLL_DATA_0;
		Freq = Freq >> 1;
		_delay_loop_1(2);
		PLL_CLK_1;
		_delay_loop_1(2);
		PLL_CLK_0;
	}
	// Stop transmition
	PLL_DLEN_0;
	PLL_CLK_0;
	PLL_DATA_0;
}

// ********* Keyboard scan functions **************
// Keyboard definition
#define KEYMENU 8
#define KEYESC 7
#define KEYMEM1 6
#define KEYMEM2 5
#define KEYMEM3 3
#define KEYMEM4 4
#define KEYMEM5 2
#define KEYMEM6 1
#define KEYUP 6
#define KEYDOWN 5
#define KEYHUP 3
#define KEYHDOWN 4
#define KEYMUP 2
#define KEYMDOWN 1
// Keyboard test - all keys
uint8_t TestKbd()
{
	uint8_t Key;

	switch(PINC & 0x3F) {
	case 0x3E: Key = 1; break;
	case 0x3D: Key = 2; break;
	case 0x3B: Key = 3; break;
	case 0x37: Key = 4; break;
	case 0x2F: Key = 5; break;
	case 0x1F: Key = 6; break;
	default: Key = 0;
	}
	if(!(PIND & 0x04)) Key = 7;
	if(!(PIND & 0x08)) Key = 8;
	return Key;
}

// Scan MENU key - return 1=pressed 
uint8_t MenuKey()
{
	if(PIND & 0x08) return 0; else return 1;
}

//*************** UART ********************
// Initialize the UART to 9600 Bd, tx/rx, 8N2
#define UART_BAUD 9600
void uart_init(void)
{
  UCSRA = _BV(U2X);					/* improve baud rate error by using 2x clk */
  UBRRL = (F_CPU / (8UL * UART_BAUD)) - 1;
  UCSRB = _BV(TXEN) | _BV(RXEN);	/* tx/rx enable */
//* frame 8 bit, 1 stop */
}


// Send character c to the UART, wait until tx holding register
void uart_putchar(char c)
{
	while(!(UCSRA & (1<<UDRE)));
	UDR = c;
}

// Get received char, (if NO then return 0)
char uart_getchar(char* pchar)
{
	if(UCSRA & (1<<RXC)) {
		*pchar=UDR;
		return 1;
	}
	else return 0;
}

// Send word to the UART
void uart_putword(uint16_t w)
{
	uart_putchar(w%256);
	uart_putchar(w/256);
}

// Get word from serial line, (max. wait time 1 sec.)
char uart_getword(uint16_t* pword)
{
	unsigned char fchar;
	int counter=10000;

	while(counter) {
		if(UCSRA & (1<<RXC)) {
			fchar=UDR;
			break;
		}
		Delay50u;
		counter--;
	}
	if(!counter) return 0;
	counter=10000;
	while(counter) {
		if(UCSRA & (1<<RXC)) break;
		Delay50u;
		counter--;
	}
	if(!counter) return 0;
	else *pword=UDR*256+fchar; return 1;
}

// ************ Power line function ***************
// Test power line
unsigned char PowerLine()
{
	unsigned char Outp;

    DDRB = 0x1F;
	Outp = PINB & 0x20;
    DDRB = 0x3F;
	return Outp;
}
